#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Tests for schooltool.app.browser.skin.
"""

import unittest
import doctest

from zope.interface import Interface, implements
from zope.app.testing import setup
from zope.publisher.browser import TestRequest
from zope.component import provideAdapter, adapts

from schooltool.testing.setup import setUpSchoolToolSite


def doctest_OrderedViewletManager_sort():
    r"""Tests for OrderedViewletManager.sort

        >>> from schooltool.skin import OrderedViewletManager

    If two viewlets have an ``order`` attribute, they are ordered by it.
    This attribute may be a string, if defined via zcml.

    If two viewlets do not have an ``order`` attribute, they are ordered
    alphabetically by their ``title`` attributes.

    If it so happens that one viewlet has an ``order`` attribute, and the other
    doesn't, the one with an order comes first.

        >>> class SomeViewlet(object):
        ...     def __init__(self, title=None, order=None):
        ...         if title is not None:
        ...             self.title = title
        ...         if order is not None:
        ...             self.order = order

        >>> mgr = OrderedViewletManager(context=None, request=None, view=None)
        >>> viewlets = [
        ...     ('name1', SomeViewlet('One', '1')),
        ...     ('name2', SomeViewlet('Apple')),
        ...     ('name3', SomeViewlet('Twenty-two', '22')),
        ...     ('name4', SomeViewlet('Five', '5')),
        ...     ('name5', SomeViewlet('Orange')),
        ...     ('name6', SomeViewlet('Grapefuit')),
        ... ]
        >>> for name, v in mgr.sort(viewlets):
        ...     print v.title
        One
        Five
        Twenty-two
        Apple
        Grapefuit
        Orange

    Viewlets may not necessarily have a title, if they have an order.

        >>> viewlets = [
        ...     ('name1', SomeViewlet(order='1')),
        ...     ('name22', SomeViewlet(order='22')),
        ...     ('name5', SomeViewlet(order='5')),
        ... ]
        >>> for name, v in mgr.sort(viewlets):
        ...     print name
        name1
        name5
        name22

    Viewlets must have either an order or a title, and the error message
    should explicitly say which viewlet is at fault:

        >>> viewlets = [
        ...     ('name1', SomeViewlet(order='1')),
        ...     ('name2', SomeViewlet()),
        ...     ('name3', SomeViewlet(title='hi')),
        ... ]
        >>> mgr.sort(viewlets)
        Traceback (most recent call last):
          ...
        AttributeError: 'name2' viewlet has neither order nor title

    """


def doctest_NavigationViewlet_appURL():
    r"""Tests for NavigationViewlet.appURL

        >>> setup.placefulSetUp()
        >>> site = setUpSchoolToolSite()

        >>> from schooltool.skin import NavigationViewlet
        >>> viewlet = NavigationViewlet()
        >>> viewlet.request = TestRequest()
        >>> viewlet.appURL()
        'http://127.0.0.1'

        >>> setup.placefulTearDown()

    """


def doctest_NavigationViewletViewCrowd():
    """Tests for NavigationViewletViewCrowd.

        >>> setup.placelessSetUp()
        >>> from schooltool.skin.skin import NavigationViewletViewCrowd

        >>> class StubCrowd(object):
        ...     def __init__(self, context):
        ...         self.context = context
        ...     def contains(self, principal):
        ...         return "Crowd on %s contains %s" % (self.context, principal)

        >>> from schooltool.securitypolicy.interfaces import ICrowd
        >>> provideAdapter(StubCrowd,
        ...                (Interface, ), ICrowd, name='schooltool.view')

        >>> class ViewletStub(object):
        ...     def actualContext(self):
        ...         return "Actual context object"

    The crowd delegates the permission check to the ICrowd based on
    the actualContext object of the viewlet:

        >>> crowd = NavigationViewletViewCrowd(ViewletStub())
        >>> crowd.contains("The principal")
        'Crowd on Actual context object contains The principal'

        >>> setup.placelessTearDown()

    """


def doctest_ActionMenuViewletManager():
    """Tests for ActionMenuViewletManager.

         >>> setup.placelessSetUp()
         >>> from schooltool.skin.skin import ActionMenuViewletManager
         >>> context = "context"
         >>> request = TestRequest()
         >>> manager = ActionMenuViewletManager(context, request, None)

    Viewlet managers title is taken from it's contexts breadcrumb info:

         >>> manager.title()
         ''

    subItems is a shortctut that gets subitems for the managers context:

         >>> manager.getSubItems = lambda context: ["Sub menu items for", context]
         >>> manager.subItems()
         ['Sub menu items for', 'context']

         >>> setup.placelessTearDown()

    """


def doctest_ActionMenuViewletManager_update():
    """Tests for ActionMenuViewletManager.update.

         >>> setup.placelessSetUp()
         >>> from schooltool.skin.skin import ActionMenuViewletManager
         >>> context = "context"
         >>> request = TestRequest()
         >>> manager = ActionMenuViewletManager(context, request, None)

    Viewlet managers title is taken from it's contexts breadcrumb info:

         >>> class TargetStub(object):
         ...      __parent__ = "parent"
         >>> target = TargetStub()
         >>> manager.getSubItems = lambda context: ["foo", "bar"]
         >>> import sys
         >>> manager.orderedViewletManagerUpdate = lambda: sys.stdout.write(
         ...                                           "OrderedViewletManager.update()")
         >>> manager.__parent__ = ActionMenuViewletManager(None, None, None)
         >>> manager.target = target

    If we are not displaying a top level menu and the current target
    is not in the list of it's parents subitems:

         >>> manager.update()
         OrderedViewletManager.update()

    Target becomes the context:

         >>> target is manager.context
         True

    If we are displaying a top level menu we should still have the
    target becoming new context:

         >>> manager.context = None
         >>> manager.update()
         OrderedViewletManager.update()

         >>> target is manager.context
         True

    But if the manager is being displayed to display a top level menu,
    and the item is in it's parents subitem list:

         >>> manager.getSubItems = lambda context: ["foo", "bar", target]
         >>> manager.__parent__ = None

         >>> manager.update()
         OrderedViewletManager.update()

    We should get menu items of the targets parent:

         >>> target.__parent__ is manager.context
         True

         >>> setup.placelessTearDown()

    """


def doctest_LanguageSelectorViewlet():
    """

         >>> from schooltool.skin.skin import LanguageSelectorViewlet
         >>> from zope.publisher.browser import TestRequest
         >>> viewlet = LanguageSelectorViewlet(None, TestRequest())

         >>> from zope.i18n.interfaces import IUserPreferredLanguages
         >>> class UPL(object):
         ...     implements(IUserPreferredLanguages)
         ...     adapts(Interface)
         ...     def __init__(self, request):
         ...         pass

     If ICookieLanguageSelector is not provided by the
     UserPreferedLanguages adapter - no languages are in the list:

         >>> provideAdapter(UPL)
         >>> viewlet.languages() is None
         True

     If adapter provides ICookieLanguageSelector viewlet returns
     whatever is given by the CookieLanguageSelector:

         >>> from zope.publisher.interfaces.http import IHTTPRequest
         >>> from schooltool.app.interfaces import ICookieLanguageSelector
         >>> class CookieLanguageSelector(UPL):
         ...     implements(ICookieLanguageSelector)
         ...     adapts(IHTTPRequest)
         ...     def getLanguageList(self):
         ...         return ["lt", "en"]
         ...     def getSelectedLanguage(self):
         ...         return "lt"
         >>> provideAdapter(CookieLanguageSelector, provides=IUserPreferredLanguages)

         >>> viewlet.languages()
         ['lt', 'en']

         >>> viewlet.selected_lang()
         'lt'

    """


def test_suite():
    return unittest.TestSuite([
                doctest.DocTestSuite(),
           ])


if __name__ == "__main__":
    unittest.main(defaultTest='test_suite')



#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2008 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Tests for person demographics
"""
import unittest
import doctest

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.basicperson.person import BasicPerson
from schooltool.basicperson.interfaces import IDemographics
from schooltool.basicperson.interfaces import IDemographicsFields
from schooltool.basicperson.demographics import DateFieldDescription
from schooltool.basicperson.demographics import EnumFieldDescription
from schooltool.basicperson.demographics import IntFieldDescription
from schooltool.basicperson.demographics import setUpDefaultDemographics
from schooltool.basicperson.demographics import IDemographicsForm
from schooltool.basicperson.demographics import TextFieldDescription
from schooltool.basicperson.demographics import BoolFieldDescription
from schooltool.basicperson.demographics import PersonDemographicsData
from schooltool.schoolyear.testing import (setUp, tearDown,
                                           provideStubUtility,
                                           provideStubAdapter)
from schooltool.basicperson.ftesting import basicperson_functional_layer


def doctest_PersonDemographicsData():
    """Tests for PersonDemographicsData

    PersonDemographicsData is a container that stores persons specific
    demographics data that users enter into the system. It is a
    persistent dict that tests for the validity of the keys before
    allowing you to write the information though so if you try writing
    to a field that was not defined in person demographics fields
    container - you will get an error.

        >>> pdd = PersonDemographicsData()
        >>> pdd['email'] = "foo@example.com"
        Traceback (most recent call last):
        ...
        InvalidKeyError: email

    On the other hand - if we add a field that is named email to the
    gradebook field storage.


        >>> app = ISchoolToolApplication(None)
        >>> IDemographicsFields(app)['email'] = TextFieldDescription("email", "Email")

    And try to set the field again - everything should work.

        >>> pdd['email'] = "foo@example.com"
        >>> pdd['email']
        'foo@example.com'

    If we try to get the information for a field that has no value
    set, we should get None:

        >>> pdd['ethnicity'] is None
        True

    On the other hand - if the field was not defined in the
    demographics fields container we will get a key error:

        >>> pdd['weight']
        Traceback (most recent call last):
        ...
        KeyError: 'weight'

    """


def doctest_getPersonDemographics():
    """Tests for Person Demographics adapter

    You should be able to adapt a person to IDemographics

        >>> app = ISchoolToolApplication(None)
        >>> john = BasicPerson("johansen", "John", "Johansen")
        >>> IDemographics(john)
        <...basicperson.demographics.PersonDemographicsData object at ...>: {}

    The demographics object is being stored in the demographics object
    container and if we try to access it again we will get the same
    object:

        >>> ddc = app['schooltool.basicperson.demographics_data']
        >>> ddc['johansen'] is IDemographics(john)
        True

    """


def doctest_removePersonDemographicsSubscriber():
    """Tests for removed person subscriber

    Add a person and ensure it has demographics

        >>> app = ISchoolToolApplication(None)
        >>> persons = app['persons']
        >>> persons['johansen'] = john = BasicPerson("johansen", "John", "Johansen")

        >>> IDemographics(john)
        <...basicperson.demographics.PersonDemographicsData object at ...>: {}

        >>> ddc = app['schooltool.basicperson.demographics_data']
        >>> ddc['johansen'] is IDemographics(john)
        True

    If we remove the person, its demographics is removed

        >>> del persons['johansen']
        >>> ddc.get('johansen', None) is None
        True

    """


def doctest_DemographicsFormAdapter():
    """Tests for DemographicsFormAdapter

    When we adapt a person into a demographics we can assign all the
    demographics attributes to the person directly.

        >>> john = BasicPerson("johansen", "John", "Johansen")
        >>> form = IDemographicsForm(john)
        >>> form.ID = "an ID"

    And it will get automatically stored in that persons demographics
    data:

        >>> IDemographics(john)['ID']
        'an ID'

    """


def doctest_setUpDefaultDemographics():
    """Tests for DemographicsFormAdapter

    If there is no demographics container in the application one get's
    created and populated automatically.

        >>> app = ISchoolToolApplication(None)
        >>> del app['schooltool.basicperson.demographics_fields']
        >>> setUpDefaultDemographics(app)

        >>> app['schooltool.basicperson.demographics_fields'].keys()
        ['ID', 'ethnicity', 'language', 'placeofbirth', 'citizenship']

    """


def doctest_DemographicsFields():
    """Tests for DemographicsFields

    DemographicsFields is a class that contains demo fields that themselves
    may or may not be limited to a group or groups.

        >>> dfs = IDemographicsFields(ISchoolToolApplication(None))
        >>> dfs['email'] = TextFieldDescription("email", "Email")
        >>> dfs['supervisor'] = TextFieldDescription("supervisor", "Supervisor",
        ...     limit_keys=['teachers'])
        >>> dfs['advisor'] = TextFieldDescription("advisor", "Advisor",
        ...     limit_keys=['students'])
        >>> dfs['phone'] = TextFieldDescription("phone", "Phone",
        ...     limit_keys=['teachers', 'students'])


    When we pass the filter_key method a key that does not belong
    to any of the limit_keys lists, then it will only return those
    fields that have empty limit_keys lists.

        >>> [f.__name__ for f in dfs.filter_key('anything')]
        [u'ID', u'ethnicity', u'language', u'placeofbirth', u'citizenship',
                u'email']

    When we pass 'teachers', it picks up the additional fields that are for
    teachers.

        >>> [f.__name__ for f in dfs.filter_key('teachers')]
        [u'ID', u'ethnicity', u'language', u'placeofbirth', u'citizenship',
                u'email', u'supervisor', u'phone']

    When we pass 'students', it picks up the additional fields that are for
    students.

        >>> [f.__name__ for f in dfs.filter_key('students')]
        [u'ID', u'ethnicity', u'language', u'placeofbirth', u'citizenship',
                u'email', u'advisor', u'phone']

    We also have a filter_keys method to return fields whose keys are in the
    list passed.

        >>> [f.__name__ for f in dfs.filter_keys([])]
        [u'ID', u'ethnicity', u'language', u'placeofbirth', u'citizenship',
                u'email']

        >>> [f.__name__ for f in dfs.filter_keys(['students'])]
        [u'ID', u'ethnicity', u'language', u'placeofbirth', u'citizenship',
                u'email', u'advisor', u'phone']

        >>> [f.__name__ for f in dfs.filter_keys(['teachers'])]
        [u'ID', u'ethnicity', u'language', u'placeofbirth', u'citizenship',
                u'email', u'supervisor', u'phone']

        >>> [f.__name__ for f in dfs.filter_keys(['students', 'teachers'])]
        [u'ID', u'ethnicity', u'language', u'placeofbirth', u'citizenship',
                u'email', u'supervisor', u'advisor', u'phone']
    """


def doctest_EnumFieldDescription():
    """Tests for EnumFieldDescription

    Enum field description is a class that defines a choice field
    shown in person add/edit form.

       >>> fd = EnumFieldDescription("ethnicity", "Ethnicity")
       >>> fd.__name__ = 'ethnicity'
       >>> fields = fd.makeField()
       >>> len(fields)
       1

       >>> field = fields['ethnicity']
       >>> field
       <Field 'ethnicity'>

       >>> field.interface
       <InterfaceClass schooltool.basicperson.demographics.IDemographicsForm>

       >>> field.field
       <zope.schema._field.Choice object at ...>

       >>> field.__name__
       'ethnicity'

    """


def doctest_IntFieldDescription():
    """Tests for IntFieldDescription

    Int field description is a class that defines minimum and maximum
    values for an integer field shown in person add/edit form.

       >>> fd = IntFieldDescription("tablets", "Tablets")
       >>> fd.__name__ = 'tablets'
       >>> fd.min_value = 2
       >>> fd.max_value = 3
       >>> fields = fd.makeField()
       >>> len(fields)
       1

       >>> field = fields['tablets']
       >>> field
       <Field 'tablets'>

       >>> field.interface
       <InterfaceClass schooltool.basicperson.demographics.IDemographicsForm>

       >>> field.field
       <zope.schema._bootstrapfields.Int object at ...>

       >>> field.__name__
       'tablets'

       >>> field.field.min
       2
       >>> field.field.max
       3

    """


def doctest_TextFieldDescription():
    """Tests for TextFieldDescription

    Enum field description is a classthat defines a text field shown
    in person add/edit form.

       >>> fd = TextFieldDescription("ID", "ID")
       >>> fd.__name__ = 'ID'
       >>> fields = fd.makeField()
       >>> len(fields)
       1

       >>> field = fields['ID']
       >>> field
       <Field 'ID'>

       >>> field.interface
       <InterfaceClass schooltool.basicperson.demographics.IDemographicsForm>

       >>> field.field
       <zope.schema._bootstrapfields.TextLine object at ...>

       >>> field.__name__
       'ID'

    """


def doctest_BoolFieldDescription():
    """Tests for BoolFieldDescription

    Boolean field description is a class that defines a boolean field shown
    in person add/edit form.

       >>> fd = BoolFieldDescription("ID", "ID")
       >>> fd.__name__ = 'ID'
       >>> fields = fd.makeField()
       >>> len(fields)
       1

       >>> field = fields['ID']
       >>> field
       <Field 'ID'>

       >>> field.interface
       <InterfaceClass schooltool.basicperson.demographics.IDemographicsForm>

       >>> field.field
       <zope.schema._bootstrapfields.Bool object at ...>

       >>> field.__name__
       'ID'

    """


def doctest_DateFieldDescription():
    """Tests for DateFieldDescription

    Date field description is a class that that defines a date field
    shown in person add/edit form.

       >>> fd = DateFieldDescription("birthday", "Birth date")
       >>> fd.__name__ = 'birthday'
       >>> fields = fd.makeField()
       >>> len(fields)
       1

       >>> field = fields['birthday']
       >>> field
       <Field 'birthday'>

       >>> field.interface
       <InterfaceClass schooltool.basicperson.demographics.IDemographicsForm>

       >>> field.field
       <zope.schema._field.Date object at ...>

       >>> field.__name__
       'birthday'

    """


def test_suite():
    optionflags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS
    suite = doctest.DocTestSuite(optionflags=optionflags,
                                 extraglobs={'provideAdapter': provideStubAdapter,
                                             'provideUtility': provideStubUtility},
                                 setUp=setUp, tearDown=tearDown)
    suite.layer = basicperson_functional_layer
    return suite


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Unit tests for course and section implementations.
"""
import unittest
import doctest
from zope.interface.verify import verifyObject

from schooltool.relationship.tests import setUp, tearDown


def doctest_CourseContainer():
    r"""Schooltool toplevel container for Courses.

        >>> from schooltool.course.interfaces import ICourseContainer
        >>> from schooltool.course.course import CourseContainer
        >>> courses = CourseContainer()
        >>> verifyObject(ICourseContainer, courses)
        True

    It should only be able to contain courses

        >>> from schooltool.course.course import Course
        >>> from schooltool.course.section import Section
        >>> from zope.container.constraints import checkObject
        >>> checkObject(courses, 'name', Course())

        >>> checkObject(courses, 'name', Section())
        Traceback (most recent call last):
          ...
        InvalidItemType: ...

    """


def doctest_Course():
    r"""Tests for Courses

    Courses are similar to SchoolTool groups but have sections instead of
    members.

        >>> from schooltool.course.course import Course
        >>> algebraI = Course("Algebra I", "First year math.")
        >>> from schooltool.course.interfaces import ICourse
        >>> verifyObject(ICourse, algebraI)
        True

    Basic properties:

        >>> algebraI.title
        'Algebra I'
        >>> algebraI.description
        'First year math.'

    Courses are instructional content that is taught in Sections, the Sections
    are related to the course with the schooltool URICourseSections
    relationship.

    To test the relationship we need to do some setup:

        >>> from schooltool.app.relationships import enforceCourseSectionConstraint
        >>> import zope.event
        >>> old_subscribers = zope.event.subscribers[:]
        >>> zope.event.subscribers.append(enforceCourseSectionConstraint)

    We need some sections and a person to test:

        >>> from schooltool.course.course import Course
        >>> from schooltool.course.section import Section
        >>> from schooltool.person.person import Person
        >>> section1 = Section(title="section1")
        >>> section2 = Section(title="section2")
        >>> section3 = Section(title="section3")
        >>> person = Person()

    Our course doesn't have any sections yet:

        >>> for section in algebraI.sections:
        ...     print section

    Lets add one:

        >>> algebraI.sections.add(section1)
        >>> for section in algebraI.sections:
        ...     print section.title
        section1

    Lets try to add a person to the course:

        >>> algebraI.sections.add(person)
        Traceback (most recent call last):
        ...
        InvalidRelationship: Sections must provide ISection.

    Lets try to add a course to the course:

        >>> history = Course()
        >>> algebraI.sections.add(history)
        Traceback (most recent call last):
        ...
        InvalidRelationship: Sections must provide ISection.

    No luck, you can only add sections:

        >>> algebraI.sections.add(section2)
        >>> algebraI.sections.add(section3)
        >>> for section in algebraI.sections:
        ...     print section.title
        section1
        section2
        section3

    That's it:

        >>> zope.event.subscribers[:] = old_subscribers

    """


def doctest_Section():
    r"""Tests for course section groups.

        >>> from schooltool.course.section import Section
        >>> section = Section(title="section 1", description="advanced")
        >>> from schooltool.course.interfaces import ISection
        >>> verifyObject(ISection, section)
        True

    sections have some basic properties:

        >>> section.title
        'section 1'
        >>> section.description
        'advanced'
        >>> section.size
        0

    We'll add an instructor to the section.

        >>> from schooltool.person.person import Person
        >>> teacher = Person('teacher', 'Mr. Jones')
        >>> section.instructors.add(teacher)

    Now we'll add some learners to the Section with the sections membership
    RelationshipProperty.

        >>> section.members.add(Person('first','First'))
        >>> section.members.add(Person('second','Second'))
        >>> section.members.add(Person('third','Third'))

        >>> for person in section.members:
        ...     print person.title
        First
        Second
        Third
        >>> section.size
        3

    We can add a Group as a member

        >>> from schooltool.group.group import Group
        >>> group = Group('group','Group')
        >>> section.members.add(group)
        >>> for member in section.members:
        ...     print member.title
        First
        Second
        Third
        group

    That group is empty so the size of our section doesn't change:

        >>> section.size
        3

    If the group grows, our section grows as well:

        >>> group.members.add(Person('fourth','Fourth'))
        >>> group.members.add(Person('fifth','Fifth'))
        >>> section.size
        5

        >>> for person in section.instructors:
        ...     print person.title
        Mr. Jones

    Sections are generally shown in the interface by their label.  Labels are
    created from the list of instructors and the section title.

        >>> from zope.i18n import translate
        >>> translate(section.label)
        u'Mr. Jones -- section 1'

    Labels are updated dynamically when more instructors are added.

        >>> section.instructors.add(Person('teacher2', 'Mrs. Smith'))
        >>> translate(section.label)
        u'Mr. Jones; Mrs. Smith -- section 1'

    The course should be listed in courses:

        >>> from schooltool.course.course import Course
        >>> course = Course(title="US History")
        >>> course.sections.add(section)
        >>> for course in section.courses:
        ...     print course.title
        US History

    Sections can be part of more than one Course:

        >>> english = Course(title="English")
        >>> section.courses.add(english)
        >>> for course in section.courses:
        ...     print course.title
        US History
        English

    """


def doctest_Section_linking():
    r"""Tests for course section linking properties (previous, next and
    linked_sections)

    The purpose of this test is to check that:
    * sections can be linked via Section.previous and Section.next.
    * Section.linked_sections return sections linked with the Section
      in a correct order.
    * it is impossible to create linking loops.

        >>> from schooltool.course.section import Section

        >>> def section_link_str(s):
        ...     return '%s <- %s -> %s' % (
        ...         not s.previous and 'None' or s.previous.title,
        ...         s.title,
        ...         not s.next and 'None' or s.next.title)

        >>> def print_sections(sections):
        ...     '''Print prev and next links for all sections in section_list'''
        ...     for s in sections:
        ...         print section_link_str(s)

        >>> def print_linked(section_list):
        ...     '''Print linked sections for all sections in section_list'''
        ...     for section in section_list:
        ...         linked_str = ', '.join(
        ...             [s.title for s in section.linked_sections])
        ...         print section.title, 'spans:', linked_str

    Create some sections.

        >>> sections = [Section('Sec0'), Section('Sec1'), Section('Sec2')]

    By default sections are not linked.

        >>> print_sections(sections)
        None <- Sec0 -> None
        None <- Sec1 -> None
        None <- Sec2 -> None

    And each section spans only itself.

        >>> print_linked(sections)
        Sec0 spans: Sec0
        Sec1 spans: Sec1
        Sec2 spans: Sec2

    Assign s0 as previous section to s1.  s0 'next' section is also updated.
    A list of linked sections updated for s0 and s1.

        >>> sections[1].previous = sections[0]

        >>> print_sections(sections)
        None <- Sec0 -> Sec1
        Sec0 <- Sec1 -> None
        None <- Sec2 -> None

        >>> print_linked(sections)
        Sec0 spans: Sec0, Sec1
        Sec1 spans: Sec0, Sec1
        Sec2 spans: Sec2

    Assign s2 as next section to s1.

        >>> sections[1].next = sections[2]

        >>> print_sections(sections)
        None <- Sec0 -> Sec1
        Sec0 <- Sec1 -> Sec2
        Sec1 <- Sec2 -> None

        >>> print_linked(sections)
        Sec0 spans: Sec0, Sec1, Sec2
        Sec1 spans: Sec0, Sec1, Sec2
        Sec2 spans: Sec0, Sec1, Sec2

    Let's test section unlinking...

        >>> sections[1].previous = None
        >>> print_sections(sections)
        None <- Sec0 -> None
        None <- Sec1 -> Sec2
        Sec1 <- Sec2 -> None

        >>> print_linked(sections)
        Sec0 spans: Sec0
        Sec1 spans: Sec1, Sec2
        Sec2 spans: Sec1, Sec2

        >>> sections[2].previous = None
        >>> print_sections(sections)
        None <- Sec0 -> None
        None <- Sec1 -> None
        None <- Sec2 -> None

        >>> print_linked(sections)
        Sec0 spans: Sec0
        Sec1 spans: Sec1
        Sec2 spans: Sec2

    And now some extreme cases.  Try the section as next/prev to itself.

        >>> sections[0].previous = sections[0]
        Traceback (most recent call last):
        ...
        InvalidSectionLinkException: Cannot assign section as previous to itself

        >>> sections[0].next = sections[0]
        Traceback (most recent call last):
        ...
        InvalidSectionLinkException: Cannot assign section as next to itself

    Create a long list of linked sections.

        >>> sections = [Section('Sec0')]
        >>> for n in range(5):
        ...     new_sec = Section('Sec%d' % (n+1))
        ...     new_sec.previous = sections[-1]
        ...     sections.append(new_sec)

        >>> print_sections(sections)
        None <- Sec0 -> Sec1
        Sec0 <- Sec1 -> Sec2
        Sec1 <- Sec2 -> Sec3
        Sec2 <- Sec3 -> Sec4
        Sec3 <- Sec4 -> Sec5
        Sec4 <- Sec5 -> None

    Try to introduce a loop by assigning a previous section.

        >>> sections[4].previous = sections[1]

    Note that sections 2 and 3 are removed from the linked list thus avoiding
    the loop.

        >>> print_sections(sections)
        None <- Sec0 -> Sec1
        Sec0 <- Sec1 -> Sec4
        None <- Sec2 -> None
        None <- Sec3 -> None
        Sec1 <- Sec4 -> Sec5
        Sec4 <- Sec5 -> None

        >>> [s.title for s in sections[0].linked_sections]
        ['Sec0', 'Sec1', 'Sec4', 'Sec5']

    Let's reubild the list of 5 linked sections.

        >>> sections[4].previous = sections[3]
        >>> sections[3].previous = sections[2]
        >>> sections[2].previous = sections[1]

        >>> print_linked(sections)
        Sec0 spans: Sec0, Sec1, Sec2, Sec3, Sec4, Sec5
        Sec1 spans: Sec0, Sec1, Sec2, Sec3, Sec4, Sec5
        Sec2 spans: Sec0, Sec1, Sec2, Sec3, Sec4, Sec5
        Sec3 spans: Sec0, Sec1, Sec2, Sec3, Sec4, Sec5
        Sec4 spans: Sec0, Sec1, Sec2, Sec3, Sec4, Sec5
        Sec5 spans: Sec0, Sec1, Sec2, Sec3, Sec4, Sec5


    Try to introduce a loop by assigning a next section.

        >>> sections[1].next = sections[4]

    Note that sections 2 and 3 are removed from the linked list thus avoiding
    the loop again.

        >>> print_sections(sections)
        None <- Sec0 -> Sec1
        Sec0 <- Sec1 -> Sec4
        None <- Sec2 -> None
        None <- Sec3 -> None
        Sec1 <- Sec4 -> Sec5
        Sec4 <- Sec5 -> None

        >>> [s.title for s in sections[0].linked_sections]
        ['Sec0', 'Sec1', 'Sec4', 'Sec5']

    Let's reubild the list of 5 linked sections.

        >>> sections[3].next = sections[4]
        >>> sections[2].next = sections[3]
        >>> sections[1].next = sections[2]

        >>> print_linked(sections)
        Sec0 spans: Sec0, Sec1, Sec2, Sec3, Sec4, Sec5
        Sec1 spans: Sec0, Sec1, Sec2, Sec3, Sec4, Sec5
        Sec2 spans: Sec0, Sec1, Sec2, Sec3, Sec4, Sec5
        Sec3 spans: Sec0, Sec1, Sec2, Sec3, Sec4, Sec5
        Sec4 spans: Sec0, Sec1, Sec2, Sec3, Sec4, Sec5
        Sec5 spans: Sec0, Sec1, Sec2, Sec3, Sec4, Sec5

    And try to introduce another loop.

        >>> sections[0].previous = sections[3]

    Sections between Sec0 and Sec3 were unlinked to avoid the loop.

        >>> print_linked(sections)
        Sec0 spans: Sec3, Sec0
        Sec1 spans: Sec1
        Sec2 spans: Sec2
        Sec3 spans: Sec3, Sec0
        Sec4 spans: Sec4, Sec5
        Sec5 spans: Sec4, Sec5

    """


def doctest_PersonInstructorCrowd():
    """Unit test for the PersonInstructorCrowd

    We'll need a section, a group, and a couple of persons:

        >>> from schooltool.course.section import Section
        >>> from schooltool.person.person import Person
        >>> section = Section(title="section 1", description="advanced")
        >>> teacher = Person('teacher', 'Mr. Jones')
        >>> p1 = Person('p1','First')
        >>> p2 = Person('p2','Second')

    Let the first pupil be a direct member of the section taught by Mr. Jones.

        >>> section.instructors.add(teacher)
        >>> section.members.add(p1)

    The PersonInstructorCrowd should contain the teacher for p1, but not p2:

        >>> from schooltool.course.section import PersonInstructorsCrowd
        >>> PersonInstructorsCrowd(p1).contains(teacher)
        True
        >>> PersonInstructorsCrowd(p2).contains(teacher)
        False

    Non-teachers are not in the crowd:

        >>> PersonInstructorsCrowd(p2).contains(p2)
        False
        >>> PersonInstructorsCrowd(p2).contains(p1)
        False

    """


def doctest_PersonLearnerAdapter(self):
    """Tests for PersonLearnerAdapter.

    We'll need a person, a group, and a couple of sections:

        >>> from schooltool.course.section import Section
        >>> from schooltool.person.person import Person
        >>> section1 = Section(title="section 1", description="advanced")
        >>> section2 = Section(title="section 2", description="advanced")
        >>> student = Person('student', 'Mr. Peter')

     Let's add the student to the two sections.

        >>> section1.members.add(student)
        >>> section2.members.add(student)

    sections method of the adapter should list us both sections:

        >>> from schooltool.course.section import PersonLearnerAdapter
        >>> learner = PersonLearnerAdapter(student)
        >>> [section.title for section in learner.sections()]
        ['section 1', 'section 2']

    """


def test_suite():
    optionflags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS
    suite = doctest.DocTestSuite(optionflags=optionflags,
                                 setUp=setUp, tearDown=tearDown)
    return suite


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2011 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
SchoolTool flourish zcml directives.
"""
import zope.browserpage.metadirectives
import zope.component.zcml
import zope.configuration.fields
import zope.schema
import zope.security.checker
import zope.viewlet.metadirectives
from zope.interface import Interface, classImplements
from zope.configuration.exceptions import ConfigurationError
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.publisher.interfaces.browser import IBrowserView

from schooltool.skin.flourish.zcml_content import contentDirective
from schooltool.skin.flourish.zcml_content import subclass_content
from schooltool.skin.flourish.zcml_content import template_specs, update_specs
from schooltool.skin.flourish.zcml_content import handle_interfaces
from schooltool.skin.flourish.zcml_content import handle_security
from schooltool.skin.flourish.zcml_content import TemplatePath
from schooltool.skin.flourish import interfaces
from schooltool.skin.flourish.page import Page
from schooltool.skin.flourish.viewlet import Viewlet, ViewletManager


class IRenderOverrides(Interface):

    update = zope.configuration.fields.PythonIdentifier(
        title=u"The name of the view attribute implementing content update.",
        required=False,
        )

    render = zope.configuration.fields.PythonIdentifier(
        title=u"The name of the view attribute that renders the content.",
        required=False,
        )


class IViewletOrder(Interface):

    after = zope.configuration.fields.Tokens(
        title=u"Display this viewlet after the specified viewlets.",
        value_type=zope.schema.TextLine(
            title=u"Names of viewlets",
            required=True),
        required=False)

    before = zope.configuration.fields.Tokens(
        title=u"Display this viewlet before the specified viewlets.",
        value_type=zope.schema.TextLine(
            title=u"Names of viewlets",
            required=True),
        required=False)


class IViewletDirective(zope.viewlet.metadirectives.IViewletDirective,
                        IRenderOverrides,
                        IViewletOrder):
    """A viewlet directive."""

    template = TemplatePath(
        title=u"Content-generating template.",
        required=False)

    manager = zope.configuration.fields.GlobalObject(
        title=u"The viewlet Manager",
        required=True)

    title = zope.configuration.fields.MessageID(
        title=u"Title of this viewlet",
        required=False,
        )


class IViewletFactoryDirective(zope.component.zcml.IAdapterDirective):

    for_ = zope.configuration.fields.GlobalObject(
        title=u"The interface or class this view is for",
        required=False,
        default=Interface,
        )

    factory = zope.configuration.fields.GlobalObject(
        title=u"The adapter factory",
        required=False,
        )

    provides = zope.configuration.fields.GlobalInterface(
        title=u"Interface the component provides",
        required=False,
        default=interfaces.IViewlet,
        )

    view = zope.configuration.fields.GlobalObject(
        title=u"The view the content provider is registered for.",
        description=u"""
        The view can either be an interface or a class. By
        default the provider is registered for all views,
        the most common case.""",
        required=False,
        default=IBrowserView,
        )

    layer = zope.configuration.fields.GlobalInterface(
        title=u"The layer the view is in",
        required=False,
        default=interfaces.IFlourishLayer,
        )

    manager = zope.configuration.fields.GlobalObject(
        title=u"The viewlet manager this viewlet is in",
        required=False,
        default=interfaces.IViewletManager,
        )


# Arbitrary keys and values are allowed to be passed to the viewlet.
IViewletDirective.setTaggedValue('keyword_arguments', True)


class IManagerDirective(zope.viewlet.metadirectives.IViewletManagerDirective,
                        IViewletOrder):
    """Viewlet manager directive."""


# Arbitrary keys and values are allowed to be passed to the manager.
IManagerDirective.setTaggedValue('keyword_arguments', True)


class IPageDirective(zope.browserpage.metadirectives.IPagesDirective,
                     IRenderOverrides):

    name = zope.schema.TextLine(
        title=u"The name of the page (view)",
        required=True,
        )

    title = zope.configuration.fields.MessageID(
        title=u"Title of this page",
        required=False,
        )

    subtitle = zope.configuration.fields.MessageID(
        title=u"Subitle of this page",
        description=u"""
        A very short description of this page.
        """,
        required=False,
        )

    template = TemplatePath(
        title=u"Main template",
        description=u"""
        Change the main template that renders everything.
        """,
        required=False,
        )

    page_template = TemplatePath(
        title=u"Template for the page",
        description=u"""
        Change template that renders the page part between the header
        and the footer.
        """,
        required=False,
        )

    content_template = TemplatePath(
        title=u"Template for the main page content.",
        description=u"""
        Set template that renders main content for this page.
        """,
        required=False,
        )


# Arbitrary keys and values are allowed to be passed to the manager.
IPageDirective.setTaggedValue('keyword_arguments', True)


class IContentOrientedDirective(Interface):

    for_ = zope.configuration.fields.GlobalObject(
        title=u"The interface or class this viewlet is active in",
        required=False,
        )

    layer = zope.configuration.fields.GlobalInterface(
        title=u"The layer",
        required=False,
        )

    view = zope.configuration.fields.GlobalObject(
        title=u"The view",
        required=False,
        )


class IActiveViewletDirective(IContentOrientedDirective):

    name = zope.schema.TextLine(
        title=u"The name of the active viewlet",
        required=False,
        )

    factory = zope.configuration.fields.GlobalObject(
        title=u"The adapter name of the active viewlet",
        required=False,
        )

    manager = zope.configuration.fields.GlobalObject(
        title=u"The viewlet manager",
        required=True,
        )


def viewletManager(
    _context, name, permission,
    for_=Interface, layer=interfaces.IFlourishLayer, view=interfaces.IPageBase,
    provides=interfaces.IViewletManager,
    class_=ViewletManager, template=None,
    update='update', render='render',
    allowed_interface=(), allowed_attributes=(),
    **kwargs):

    bases = (class_, )
    if not interfaces.IViewletManager.implementedBy(class_):
        bases = bases + (ViewletManager, )
    class_ = type(class_.__name__, bases, {})

    allowed_interface = (tuple(allowed_interface) +
                         (interfaces.IViewletManager, ))

    if not provides.implementedBy(class_):
        classImplements(class_, provides)

    contentDirective(
        _context, name, permission,
        for_=for_, layer=layer, view=view,
        class_=class_, template=template,
        update=update, render=render,
        allowed_interface=allowed_interface,
        allowed_attributes=allowed_attributes,
        **kwargs)


def viewlet(
    _context, name, permission,
    for_=Interface, layer=interfaces.IFlourishLayer, view=IBrowserView,
    manager=None,
    class_=Viewlet, template=None,
    update='update', render='render',
    allowed_interface=(), allowed_attributes=(),
    **kwargs):

    if not interfaces.IViewlet.implementedBy(class_):
        class_ = type(class_.__name__, (class_, Viewlet), {})
    allowed_interface = (tuple(allowed_interface) +
                         (interfaces.IViewlet, ))

    if (render == 'render' and
        class_.render == Viewlet.render):
        if template:
            render = 'template'
        else:
            raise ConfigurationError("When template and render not specified, "
                                     "class must implement 'render' method")

    class_ = subclass_content(
        class_, name,
        {'update': update, 'render': render},
        update_specs({'template': template}, view), kwargs)

    handle_interfaces(_context, (for_, view))
    handle_interfaces(_context, allowed_interface)

    handle_security(class_, permission,
                    allowed_interface, allowed_attributes)

    _context.action(
        discriminator=('schooltool.skin.flourish.viewlet',
                       for_, layer, view, manager, name),
        callable=zope.component.zcml.handler,
        args=('registerAdapter',
              class_,
              (for_, layer, view, manager),
              interfaces.IViewlet,
              name,
              _context.info),)


def viewletFactory(
    _context, factory, provides=interfaces.IViewlet,
    for_=Interface, permission=None,
    name='', trusted=True, locate=False,
    layer=interfaces.IFlourishLayer,
    view=IBrowserView,
    manager=interfaces.IViewletManager):

    wrapper = lambda c, r, v, m: factory(c, r, v, m, name)
    objects = [for_, layer, view, manager]
    zope.component.zcml.adapter(
        _context, [wrapper],
        provides=provides, for_=objects, permission=permission,
        name=name,
        trusted=trusted, locate=locate)


def page(_context, name, permission,
         for_=Interface, layer=interfaces.IFlourishLayer,
         title=None, subtitle=None,
         template=None, page_template=None, content_template=None,
         class_=Page,
         update='update', render='render',
         allowed_interface=(), allowed_attributes=(),
         **kwargs
         ):

    forward_methods = {
        'update': update,
        'render': render,
        }

    # BBB: add index to ease porting from old style views
    if (IBrowserView.implementedBy(class_) and
        getattr(class_, 'index', None) is None):
        forward_methods['index'] = render

    if not interfaces.IPage.implementedBy(class_):
        class_ = type(class_.__name__, (class_, Page), {})

    allowed_interface = (tuple(allowed_interface) +
                         (interfaces.IPage, ))

    class_dict = dict(kwargs)
    class_dict['__name__'] = name

    if title is not None:
        class_dict['title'] = title
    if subtitle is not None:
        class_dict['subtitle'] = subtitle

    # XXX: raise ConfigurationError if class_ is Page and
    #      no templates specified

    templates = template_specs({
        'template': template,
        'page_template': page_template,
        'content_template': content_template,
        }, content_type='html')

    class_ = subclass_content(
        class_, name,
        forward_methods,
        templates,
        class_dict,
        )

    handle_interfaces(_context, (for_,))
    handle_interfaces(_context, allowed_interface)

    handle_security(class_, permission,
                    allowed_interface, allowed_attributes)

    _context.action(
        discriminator=('view', for_, name, IBrowserRequest, layer),
        callable=zope.component.zcml.handler,
        args=('registerAdapter',
              class_, (for_, layer), Interface, name, _context.info),
        )


def activeViewlet(_context, name=None, factory=None,
                  for_=Interface, layer=interfaces.IFlourishLayer,
                  view=IBrowserView, manager=interfaces.IViewletManager):

    if name is not None and factory is not None:
        raise ConfigurationError("name and factory are mutually exclusive.")

    if name is not None:
        factory = lambda *args: name

    _context.action(
        discriminator=('flourish activeViewlet',
                       for_, layer, view, manager),
        callable=zope.component.zcml.handler,
        args=('registerAdapter',
              factory, (for_, layer, view, manager),
              interfaces.IActiveViewletName, '', _context.info),
        )



#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2011 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
SchoolTool calendar overlay views.
"""

from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
from zope.component import getUtility
from zope.security.proxy import removeSecurityProxy

import schooltool.skin.flourish.page
from schooltool.app.browser.overlay import CalendarOverlayBase
from schooltool.common import DateRange
from schooltool.skin import flourish
from schooltool.term.interfaces import IDateManager
from schooltool.term.interfaces import ITerm

from schooltool.common import SchoolToolMessage as _


class CalendarOverlayView(flourish.page.Refine, CalendarOverlayBase):
    title = _('Show')
    body_template = ViewPageTemplateFile('templates/calendar_overlay.pt')
    update = CalendarOverlayBase.update

    def render(self, *args, **kw):
        if not self.show_overlay():
            return ''
        return flourish.page.Refine.render(self, *args, **kw)

    def grouped_items(self):
        # XXX: this introduces dependency on terms.  A generic grouping
        #      would be more appropriate.
        items = super(CalendarOverlayView, self).items()
        non_term_items = []
        by_term = {}
        current_term = removeSecurityProxy(
            getUtility(IDateManager).current_term)

        for item in items:
            term = ITerm(item['calendar'].__parent__, None)
            unsecure_term = removeSecurityProxy(term)
            if unsecure_term is None:
                non_term_items.append(item)
                continue
            term_range = DateRange(term.first, term.last)
            view_range = self.view.cursor_range
            if term_range.overlaps(view_range):
                if unsecure_term not in by_term:
                    by_term[unsecure_term] = {
                        'group': term,
                        'items': [],
                        'expanded': unsecure_term is current_term,
                        }
                by_term[unsecure_term]['items'].append(item)

        order = sorted(by_term, key=lambda t: t.last, reverse=True)
        result = []
        if non_term_items:
            result.append({
                    'group': None,
                    'items': non_term_items,
                    'expanded': True,
                    })
        for term in order:
            result.append(by_term[term])
        return result




#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2010 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Upgrade SchoolTool to generation 36.
"""
from zope.app.generations.utility import findObjectsProviding
from zope.app.publication.zopepublication import ZopePublication
from zope.component.hooks import getSite, setSite

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.generations.evolve36.evolve import evolveTimetables


def evolve(context):
    root = context.connection.root().get(ZopePublication.root_name, None)

    # Invalidate connection cache
    # Earlier evolutions (like evolve34) may have cached broken
    # persistent objects and we'll want to reload them.
    cache = context.connection._cache
    cache.invalidate(cache.cache_data.keys())

    old_site = getSite()

    apps = findObjectsProviding(root, ISchoolToolApplication)
    for app in apps:
        setSite(app)
        evolveTimetables(app)

    setSite(old_site)



#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
HTML Analyzation Tools
"""
from lxml import etree


def to_string(node):
    if isinstance(node, basestring):
        return node
    else:
        return etree.tostring(node, pretty_print=True)


def queryHTML(xpath, response):
    """Helper function to perform an xpath query on an html response"""
    # Emulate the (annoying) behaviour of libxml2 < 2.9.0
    # https://bugzilla.gnome.org/show_bug.cgi?id=681822
    parser = etree.HTMLParser(remove_blank_text=True)
    doc = etree.HTML(response, parser)
    result = [to_string(node) for node in doc.xpath(xpath)]
    return result


def printQuery(xpath, response):
    """Helper function to print xpath query results on an html response"""
    for result in queryHTML(xpath, response):
        if result.strip():
            print result




#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2008 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Tests for schooltool term views.
"""
import unittest
import doctest
from datetime import date

from zope.interface import directlyProvides
from zope.publisher.browser import TestRequest

from schooltool.term.term import Term
from schooltool.term.ftesting import term_functional_layer
from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.skin.skin import ISchoolToolSkin
from schooltool.schoolyear.interfaces import ISchoolYearContainer
from schooltool.schoolyear.schoolyear import SchoolYear
from schooltool.schoolyear.testing import provideStubAdapter
from schooltool.schoolyear.testing import provideStubUtility
from schooltool.schoolyear.testing import setUp
from schooltool.schoolyear.testing import tearDown
from schooltool.term.browser.term import TermEditForm
from schooltool.term.browser.term import TermAddForm


def doctest_TermAddForm():
    """Test for term add view.

    Let's add a school year

        >>> app = ISchoolToolApplication(None)
        >>> syc = ISchoolYearContainer(app)
        >>> sy = syc['2005-2006'] = SchoolYear("2005-2006",
        ...                               date(2005, 9, 1),
        ...                               date(2006, 7, 16))

    Fill in the form

        >>> request = TestRequest()
        >>> directlyProvides(request, [ISchoolToolSkin])
        >>> request.form = {'form.buttons.add' : u'Add',
        ...                 'form.widgets.title': u'Fall',
        ...                 'form.widgets.first': u'2005-09-01',
        ...                 'form.widgets.last' : u'2005-12-31'}

    And submit it

        >>> view = TermAddForm(sy, request)
        >>> view.update()
        >>> view.widgets.errors
        ()

    Now we can look at our newly added term

        >>> term = sy['fall']
        >>> term.title, term.first, term.last
        (u'Fall', datetime.date(2005, 9, 1), datetime.date(2005, 12, 31))

    """


def doctest_TermAddForm_overlap():
    """Test for term add view when the new term overlaps an old one.

     Let's add a school year

        >>> app = ISchoolToolApplication(None)
        >>> syc = ISchoolYearContainer(app)
        >>> sy = syc['2005-2006'] = SchoolYear("2005-2006",
        ...                                    date(2005, 9, 1),
        ...                                    date(2006, 7, 16))

     And one term

        >>> sy['fall'] = Term("Fall",
        ...                   date(2005, 9, 1),
        ...                   date(2006, 7, 16))

     Fill in the form providing a date range that overlaps the term we
     just added

        >>> request = TestRequest()
        >>> directlyProvides(request, [ISchoolToolSkin])
        >>> request.form = {'form.buttons.next' : u'Next',
        ...                 'form.widgets.title': u'Spring',
        ...                 'form.widgets.first': u'2005-09-01',
        ...                 'form.widgets.last' : u'2005-12-31'}

     And submit the form

        >>> view = TermAddForm(sy, request)
        >>> view.update()

     We should get an error message

        >>> for error in view.widgets.errors: print error.render()
        <div class="error">
        Date range you have selected overlaps with the following term(s):
        <div> <a href="http://127.0.0.1/schoolyears/2005-2006/fall">Fall</a> (2005-09-01 &mdash; 2006-07-16) </div>
        </div>

     And only have the old term in the school year:

        >>> list(sy.keys())
        [u'fall']

    """


def doctest_TermAddForm_out_of_bounds():
    """Test for term add view when adding a term that does not fit in the school year.

    Let's add a school year

       >>> app = ISchoolToolApplication(None)
       >>> syc = ISchoolYearContainer(app)
       >>> sy = syc['2005-2006'] = SchoolYear("2005-2006",
       ...                                    date(2005, 9, 1),
       ...                                    date(2006, 7, 16))

    Fill in the form

       >>> request = TestRequest()
       >>> directlyProvides(request, [ISchoolToolSkin])
       >>> request.form = {'form.buttons.next' : u'Next',
       ...                 'form.widgets.title': u'Spring',
       ...                 'form.widgets.first': u'2005-08-01',
       ...                 'form.widgets.last' : u'2005-12-31'}

    And submit it

       >>> view = TermAddForm(sy, request)
       >>> view.update()

    We should get an error

       >>> for error in view.widgets.errors: print error.render()
       <div class="error">Date is not in the school year.</div>

    The error should be displayed on the widget as well

       >>> print view.widgets['first'].error.render()
       <div class="error">Date is not in the school year.</div>

    There should be no terms in the container

       >>> list(sy.keys())
       []

    """


def doctest_TermEditForm_overlap():
    """Test for term editing form when making one term overlapping another.

    Let's add a school year

        >>> app = ISchoolToolApplication(None)
        >>> syc = ISchoolYearContainer(app)
        >>> sy = syc['2005-2006'] = SchoolYear("2005-2006",
        ...                                    date(2005, 9, 1),
        ...                                    date(2006, 7, 16))

    And a couple of terms

        >>> sy['fall'] = Term("Fall",
        ...                   date(2005, 9, 1),
        ...                   date(2006, 1, 1))

        >>> term = sy['spring'] = Term("Spring",
        ...                            date(2006, 1, 2),
        ...                            date(2006, 7, 16))

    Fill in the form

        >>> request = TestRequest()
        >>> directlyProvides(request, [ISchoolToolSkin])
        >>> request.form = {'form.buttons.apply': u'Save changes',
        ...                 'form.widgets.title': u'Spring',
        ...                 'form.widgets.first': u'2006-01-01',
        ...                 'form.widgets.last' : u'2006-07-16'}
        >>> view = TermEditForm(term, request)
        >>> view.update()

    We should get an error

        >>> for error in view.widgets.errors: print error.render()
        <div class="error">
        Date range you have selected overlaps with the following term(s):
        <div> <a href="http://127.0.0.1/schoolyears/2005-2006/fall">Fall</a> (2005-09-01 &mdash; 2006-01-01) </div>
        </div>

    The term should not get modified

        >>> term.first, term.last, term.title
        (datetime.date(2006, 1, 2), datetime.date(2006, 7, 16), 'Spring')

    """


def doctest_TermEditForm_overflow():
    """Test term editing form when making a term overflow a school year.

    Let's add a school year

        >>> app = ISchoolToolApplication(None)
        >>> syc = ISchoolYearContainer(app)
        >>> sy = syc['2005-2006'] = SchoolYear("2005-2006",
        ...                                    date(2005, 9, 1),
        ...                                    date(2006, 7, 16))

    And a term

        >>> term = sy['spring'] = Term("Spring",
        ...                            date(2006, 1, 2),
        ...                            date(2006, 7, 16))

    Fill in the form

        >>> request = TestRequest()
        >>> directlyProvides(request, [ISchoolToolSkin])
        >>> request.form = {'form.buttons.apply': u'Save changes',
        ...                 'form.widgets.title': u'Spring',
        ...                 'form.widgets.first': u'2006-01-02',
        ...                 'form.widgets.last' : u'2006-07-17'}

    And submit it

        >>> view = TermEditForm(term, request)
        >>> view.update()

    We should get an error

        >>> for error in view.widgets.errors: print error.render()
        <div class="error">Date is not in the school year.</div>

    And term should stay the same

        >>> term.first, term.last, term.title
        (datetime.date(2006, 1, 2), datetime.date(2006, 7, 16), 'Spring')

    """


def doctest_TermEditForm_switch_dates():
    """Test term editing form when making a term with first date after the last one.

    Let's add a school year

        >>> app = ISchoolToolApplication(None)
        >>> syc = ISchoolYearContainer(app)
        >>> sy = syc['2005-2006'] = SchoolYear("2005-2006",
        ...                                    date(2005, 9, 1),
        ...                                    date(2006, 7, 16))

    And add a term

        >>> term = sy['spring'] = Term("Spring",
        ...                            date(2006, 1, 2),
        ...                            date(2006, 7, 16))

    Fill in the form switching start and end dates

        >>> request = TestRequest()
        >>> directlyProvides(request, [ISchoolToolSkin])
        >>> request.form = {'form.buttons.apply': u'Save changes',
        ...                 'form.widgets.title': u'Spring',
        ...                 'form.widgets.first': u'2006-07-10',
        ...                 'form.widgets.last' : u'2006-01-02'}

    And submit it

        >>> view = TermEditForm(term, request)
        >>> view.update()

    We should get an error

        >>> for error in view.widgets.errors: print error.render()
        <div class="error">Term must begin before it ends.</div>

    And term should stay the same

        >>> term.first, term.last, term.title
        (datetime.date(2006, 1, 2), datetime.date(2006, 7, 16), 'Spring')

    """


def test_suite():
    optionflags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS
    suite = doctest.DocTestSuite(optionflags=optionflags,
                                 extraglobs={'provideAdapter': provideStubAdapter,
                                             'provideUtility': provideStubUtility},
                                 setUp=setUp, tearDown=tearDown)
    suite.layer = term_functional_layer
    return suite


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""Table tests
"""
import unittest
import doctest

from zope.interface import implements, Interface
from zope.component import provideAdapter
from zope.publisher.browser import TestRequest
from zope.traversing.browser.interfaces import IAbsoluteURL

from schooltool.app.browser.testing import setUp, tearDown


class FormatterStub(object):
    def __init__(self):
        self.request = TestRequest()


def doctest_FilterWidget():
    """Doctest for FilterWidget.

    Let's create the FilterWidget:

        >>> from zope.publisher.browser import TestRequest
        >>> from schooltool.table.table import FilterWidget
        >>> class ItemStub(object):
        ...     def __init__(self, title):
        ...         self.title = title
        ...     def __repr__(self):
        ...         return '<ItemStub %s>' % self.title
        >>> container = map(ItemStub, ['alpha', 'beta', 'lambda'])
        >>> request = TestRequest()
        >>> widget = FilterWidget(container, request)

    Rendering the widget returns whatever is returned by calling the template:

        >>> widget.template = lambda : 'I am a rendered widget!'
        >>> widget.render()
        'I am a rendered widget!'

    The state of the widget (whether it will filter the data or not)
    is determined by checking whether there is a query parameter in
    the request:

        >>> widget.active()
        False

        >>> request.form = {'SEARCH': 'lamb'}
        >>> widget.active()
        True

    The information that we got from the request can be appended to
    the url:

        >>> widget.extra_url()
        '&SEARCH=lamb'

    If there is no query in the request - an empty string get seturned:

        >>> request.form = {}
        >>> widget.extra_url()
        ''

    Filtering is done by skipping any entry that doesn't contain the
    query string in it's title:

        >>> request.form = {'SEARCH': 'lamb'}
        >>> widget.filter(widget.context)
        [<ItemStub lambda>]

   The search is case insensitive:

        >>> request.form = {'SEARCH': 'AlphA'}
        >>> widget.filter(widget.context)
        [<ItemStub alpha>]

    If clear search button is clicked, the form attribute is cleared,
    and all items are displayed:

        >>> request.form['CLEAR_SEARCH'] = 'Yes'

        >>> widget.filter(widget.context)
        [<ItemStub alpha>, <ItemStub beta>, <ItemStub lambda>]
        >>> request.form['SEARCH']
        ''
    """


def doctest_CheckboxColumn():
    """Tests for CheckboxColumn.

    Let's try creating a CheckboxColumn first:

        >>> from schooltool.table.table import CheckboxColumn
        >>> column = CheckboxColumn("prefix", "name", "title")
        >>> column.title
        'title'
        >>> column.name
        'name'
        >>> column.prefix
        'prefix'

        >>> class ItemStub(object):
        ...     __name__ = "itemStub"
        >>> column.renderCell(ItemStub(), None)
        '<input type="checkbox" name="prefix.itemStub" id="prefix.itemStub" />'

    """


def doctest_label_cell_formatter_factory():
    """Tests for label_cell_formatter_factory.

    Let's create a label formatter:

        >>> from schooltool.table.table import label_cell_formatter_factory
        >>> formatter = label_cell_formatter_factory()

    And render it:

        >>> class ItemStub(object):
        ...     __name__ = "item_stub"

        >>> formatter("Title of The Item", ItemStub(), "formatter")
        '<label for="item_stub">Title of The Item</label>'

    If we pass a prefix to the formatter factory, we get a label with
    the prefix before the item id:

        >>> formatter = label_cell_formatter_factory("some_prefix")
        >>> formatter("Title of The Item", ItemStub(), "formatter")
        '<label for="some_prefix.item_stub">Title of The Item</label>'

    """


def doctest_LocaleAwareGetterColumn():
    """Tests for LocaleAwareGetterColumn.

    Create a formatter with a request:

        >>> from zope.publisher.browser import TestRequest
        >>> request = TestRequest()
        >>> class FormatterStub(object):
        ...     request = request
        >>> formatter = FormatterStub()

    Register collation adapter:

        >>> from zope.i18n.interfaces.locales import ICollator
        >>> from zope.i18n.interfaces.locales import ILocale
        >>> from zope.component import adapts
        >>> class CollatorAdapterStub(object):
        ...     implements(ICollator)
        ...     adapts(ILocale)
        ...     def __init__(self, context):
        ...         self.context = context
        ...     def key(self, string):
        ...         return "CollatorKey(%s)" % string
        >>> provideAdapter(CollatorAdapterStub)

    Now when we try to get the sort key instead of getting the
    attribute of the object, we will get a collator key:

        >>> from schooltool.table.table import LocaleAwareGetterColumn
        >>> lac = LocaleAwareGetterColumn()
        >>> item = "Item"
        >>> lac.getSortKey(item, formatter)
        'CollatorKey(Item)'

    """


def doctest_ImageInputColumn():
    """Tests for ImageInputColumn.

    Let's try creating a ImageInputColumn first:

        >>> from schooltool.table.table import ImageInputColumn
        >>> column = ImageInputColumn("prefix", title="title",
        ...                           alt="alt", image="image.png")

        >>> column.title
        'title'
        >>> column.alt
        'alt'
        >>> column.name
        'title'
        >>> column.prefix
        'prefix'
        >>> column.image
        'image.png'

        >>> class ItemStub(object):
        ...     def __init__(self, name="itemStub"):
        ...         self.__name__ = name

        >>> from schooltool.common import CommonResourceURL
        >>> provideAdapter(CommonResourceURL)

    When resources are not found, column will render empty.

        >>> column.renderCell(ItemStub(), FormatterStub())
        ''

    Let's register a resource and a library with one resource.

        >>> class IStubResource(Interface):
        ...     pass

        >>> class ImageStub(object):
        ...     implements(IStubResource)
        ...     __parent__ = None
        ...     def __init__(self, name):
        ...         self.name = name
        ...
        ...     def __repr__(self):
        ...         return '<%s %r>' % (self.__class__.__name__, self.name)

        >>> class LibraryStub(object):
        ...     implements(IStubResource)
        ...     def __init__(self, name, resources):
        ...         self.name = name
        ...         self.resources = dict([(r.name, r) for r in resources])
        ...         for r in resources:
        ...             r.__parent__ = self
        ...
        ...     def get(self, name):
        ...         return self.resources.get(name)

        >>> def resourceURL(resource, request):
        ...     url = resource.name
        ...     if resource.__parent__:
        ...         url = '%s/%s' % (resource.__parent__.name, url)
        ...     return lambda: 'http://localhost/@@/%s' % url

        >>> def provideResource(resource):
        ...     provideAdapter(lambda x: resource, (TestRequest,),
        ...                    provides=Interface,
        ...                    name=resource.name)

        >>> image = ImageStub('image.png')
        >>> image2 = ImageStub('hello.png')
        >>> library = LibraryStub('images', [image2])

        >>> provideResource(image)
        >>> provideResource(library)

        >>> provideAdapter(resourceURL, (IStubResource, TestRequest),
        ...                provides=IAbsoluteURL)


    Now we can render our image directly.

        >>> print column.renderCell(ItemStub(), FormatterStub()) # doctest: +SKIP
        <button class="image" type="submit" name="prefix.itemStub"
                title="title" value="1">
          <img src="http://localhost/@@/image.png" alt="alt" />
        </button>

    Or have a column that renders an image from a library.

        >>> column = ImageInputColumn("prefix", title="title",
        ...                           alt="alt", library="images",
        ...                           image="hello.png")

        >>> print column.renderCell(ItemStub('otherItem'), FormatterStub())
        <button class="image" type="submit" name="prefix.otherItem"
                title="title" value="1">
          <img src="http://localhost/@@/images/hello.png" alt="alt" />
        </button>

    """


def doctest_SchoolToolTableFormatter():
    """Tests for SchoolToolTableFormatter.

    First some set up so we could render actual tables:

        >>> from zope.interface import Interface
        >>> class ResourcePathStub(object):
        ...     def __init__(self, context):
        ...         pass
        ...     def __call__(self):
        ...         return ""
        >>> provideAdapter(ResourcePathStub,
        ...                adapts=[Interface],
        ...                provides=Interface,
        ...                name="zc.table")

    A table formatter is meant to make rendering of tables more
    standard and convenient. All you need to do is create a table
    formatter, though usualy it is an adapter on the container and
    request, but for the testing purposes we will create it directly:

        >>> from schooltool.table.table import SchoolToolTableFormatter
        >>> container = {}
        >>> from zope.publisher.browser import TestRequest
        >>> request = TestRequest()
        >>> formatter = SchoolToolTableFormatter(container, request)

        >>> from schooltool.table.interfaces import ITableFormatter
        >>> from zope.interface.verify import verifyObject
        >>> verifyObject(ITableFormatter, formatter)
        True

    Before rendering the table we must set it up, we will provide a
    fake filter function so we would not have to set up FilterWidgets
    yet:

        >>> formatter.setUp()

        >>> print formatter.render()
        <BLANKLINE>
        <table class="data">
          <thead>
            <tr>
              <th class="title">
        <BLANKLINE>
                    <span class="zc-table-sortable zc-table-sort-asc-primary"
                          onclick="javascript: onSortClickForm(
                                'title', '.sort_on')">
                        Title</span>
        <BLANKLINE>
              </th>
            </tr>
          </thead>
          <tbody>
          </tbody>
        </table>
        <input type="hidden" name=".sort_on:tokens" id=".sort_on" value="title" />
        <BLANKLINE>

    The table has no items though, but these items are sorted by title
    by default.


    Lets add some items to play around with:

        >>> class ItemStub(object):
        ...     def __init__(self, title, email):
        ...         self.title, self.email = title, email
        ...         self.__name__ = self.title.lower()

        >>> pete = container['pete'] = ItemStub('Pete', 'pete@example.com')
        >>> john = container['john'] = ItemStub('John', 'john@example.com')
        >>> toad = container['toad'] = ItemStub('Toad', 'toad@example.com')
        >>> frog = container['frog'] = ItemStub('Frog', 'frog@example.com')

    Now the table has all 4 of them:

        >>> formatter.setUp()
        >>> print formatter.render()
        <BLANKLINE>
        ...
        <tbody>
        <tr class="odd">
          <td>
            Frog
          </td>
        </tr>
        <tr class="even">
          <td>
            John
          </td>
        </tr>
        <tr class="odd">
          <td>
            Pete
          </td>
        </tr>
        <tr class="even">
          <td>
            Toad
          </td>
        </tr>
        </tbody>
        ...

    Lets provide a filter widget:

        >>> from schooltool.table.interfaces import IFilterWidget
        >>> class FilterWidget(object):
        ...     implements(IFilterWidget)
        ...     def __init__(self, context, request):
        ...         self.request = request
        ...     def filter(self, list):
        ...         if 'SEARCH' in self.request:
        ...             return [item for item in list
        ...                     if self.request['SEARCH'] in item.title.lower()]
        ...         return list
        ...     def extra_url(self):
        ...         return "&search_info"

        >>> from zope.interface import Interface
        >>> from zope.publisher.interfaces.browser import IBrowserRequest
        >>> provideAdapter(FilterWidget, adapts=[Interface, IBrowserRequest],
        ...                              provides=IFilterWidget)

        >>> formatter = SchoolToolTableFormatter(container, request)

    Now if we will put a SEARCH string into the request, we will only
    get part of the items:

        >>> request.form = {'SEARCH': 'o'}
        >>> formatter.setUp()
        >>> print formatter.render()
        <BLANKLINE>
        ...
        <tbody>
        <tr class="odd">
          <td>
            Frog
          </td>
        </tr>
        <tr class="even">
          <td>
            John
          </td>
        </tr>
        <tr class="odd">
          <td>
            Toad
          </td>
        </tr>
        </tbody>
        ...

    We can limit items that are being displayed ourselves as well:

        >>> request.form = {}
        >>> formatter.setUp(items=[frog, toad])
        >>> print formatter.render()
        <BLANKLINE>
        ...
        <tbody>
        <tr class="odd">
          <td>
            Frog
          </td>
        </tr>
        <tr class="even">
          <td>
            Toad
          </td>
        </tr>
        </tbody>
        ...

    We can provide custom columns to the formatter if we want:

        >>> from zc.table.column import GetterColumn
        >>> email = GetterColumn(name='email',
        ...                      title=u"Email",
        ...                      getter=lambda i, f: i.email,
        ...                      subsort=True)
        >>> from zope.interface import directlyProvides
        >>> from zc.table.interfaces import ISortableColumn
        >>> directlyProvides(email, ISortableColumn)
        >>> formatter.setUp(columns=[email], sort_on=(("email", False),))
        >>> print formatter.render()
        <BLANKLINE>
        ...
        <tbody>
        <tr class="odd">
          <td>
            frog@example.com
          </td>
        </tr>
        <tr class="even">
          <td>
            john@example.com
          </td>
        </tr>
        <tr class="odd">
          <td>
            pete@example.com
          </td>
        </tr>
        <tr class="even">
          <td>
            toad@example.com
          </td>
        </tr>
        </tbody>
        ...

    We could of course add the column after the title:

        >>> formatter.setUp(columns_after=[email])
        >>> print formatter.render()
        <BLANKLINE>
        ...
        <tbody>
        <tr class="odd">
          <td>
            Frog
          </td>
          <td>
            frog@example.com
          </td>
        </tr>
        <tr class="even">
          <td>
            John
          </td>
          <td>
            john@example.com
          </td>
        </tr>
        <tr class="odd">
          <td>
            Pete
          </td>
          <td>
            pete@example.com
          </td>
        </tr>
        <tr class="even">
          <td>
            Toad
          </td>
          <td>
            toad@example.com
          </td>
        </tr>
        </tbody>
        ...

    Now let's add a checkbox before the Title column:

        >>> from schooltool.table.table import CheckboxColumn
        >>> checkbox = CheckboxColumn("delete", "delete", "Delete")
        >>> formatter.setUp(columns_before=[checkbox], columns_after=[email],
        ...                 items=[frog, john])
        >>> print formatter.render()
        <BLANKLINE>
        ...
        <tbody>
        <tr class="odd">
          <td>
            <input type="checkbox" name="delete.frog" id="delete.frog" />
          </td>
          <td>
            Frog
          </td>
          <td>
            frog@example.com
          </td>
        </tr>
        <tr class="even">
          <td>
            <input type="checkbox" name="delete.john" id="delete.john" />
          </td>
          <td>
            John
          </td>
          <td>
            john@example.com
          </td>
        </tr>
        </tbody>
        ...

    We can pass custom formatters for the default columns if we want:

        >>> from schooltool.table.table import label_cell_formatter_factory
        >>> label_formatter = label_cell_formatter_factory("delete")
        >>> formatter.setUp(columns_before=[checkbox], columns_after=[email],
        ...                 items=[frog, john], formatters=[label_formatter])
        >>> print formatter.render()
        <BLANKLINE>
        ...
        <tbody>
        <tr class="odd">
          <td>
            <input type="checkbox" name="delete.frog" id="delete.frog" />
          </td>
          <td>
            <label for="delete.frog">Frog</label>
          </td>
          <td>
            frog@example.com
          </td>
        </tr>
        <tr class="even">
          <td>
            <input type="checkbox" name="delete.john" id="delete.john" />
          </td>
          <td>
            <label for="delete.john">John</label>
          </td>
          <td>
            john@example.com
          </td>
        </tr>
        </tbody>
        ...

    We can alter the batch size and starting point through the request

        >>> request.form = {'batch_start': '2',
        ...                 'batch_size': '2'}

        >>> formatter = SchoolToolTableFormatter(container, request)
        >>> formatter.setUp()

        >>> print formatter.render()
        <BLANKLINE>
        <table class="data">
          <thead>
            <tr>
              <th class="title">
        <BLANKLINE>
                    <span class="zc-table-sortable zc-table-sort-asc-primary" ...>
                        Title</span>
        <BLANKLINE>
              </th>
            </tr>
          </thead>
          <tbody>
          <tr class="odd">
            <td>
              Pete
            </td>
          </tr>
          <tr class="even">
            <td>
              Toad
            </td>
          </tr>
          </tbody>
        </table>
        <input type="hidden" name=".sort_on:tokens" id=".sort_on" value="title" />
        <BLANKLINE>

    """


def test_suite():
    optionflags = (doctest.ELLIPSIS | doctest.REPORT_NDIFF
                   | doctest.REPORT_ONLY_FIRST_FAILURE
                   | doctest.NORMALIZE_WHITESPACE)
    suite = unittest.TestSuite()
    suite.addTest(doctest.DocTestSuite(setUp=setUp, tearDown=tearDown,
                                       optionflags=optionflags))
    return suite


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2012 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


def registerSeleniumSetup():
    try:
        import selenium
    except ImportError:
        return
    from schooltool.testing import registry
    import schooltool.testing.selenium
    import selenium.webdriver.common.keys
    from selenium.webdriver.support.select import Select

    def type_in_date(element, date):
        keys = selenium.webdriver.common.keys.Keys
        element.type(keys.DELETE, date, keys.ENTER)
        browser = element.browser
        if browser is not None:
            browser.wait_no(lambda: browser.query.id('ui-datepicker-div').is_displayed())

    registry.register('SeleniumHelpers',
        lambda: schooltool.testing.selenium.registerElementUI('enter_date',
                                                              type_in_date))

    def select_option(element, option):
        select = Select(element)
        select.select_by_visible_text(option)

    registry.register('SeleniumHelpers',
        lambda: schooltool.testing.selenium.registerElementUI('select_option',
                                                              select_option))

    def set_value(element, value):
        tag = element.tag_name
        css_class = element.get_attribute('class')
        if 'date-field' in css_class:
            element.ui.enter_date(value)
        elif tag in ('select',):
            element.ui.select_option(value)
        else:
            element.type(value)

    registry.register('SeleniumHelpers',
        lambda: schooltool.testing.selenium.registerElementUI('set_value',
                                                              set_value))

registerSeleniumSetup()
del registerSeleniumSetup
